from AccessControl import ClassSecurityInfo
from Acquisition import Implicit
from Globals import InitializeClass

def genKey(fkey, lkey) :
    return fkey + chr(lkey + 65)

def isChild(father, key) :
    return key.startswith(father)

class Comment(Implicit) :
    """
    This class stores comments and allow the nesting of comment.
    """

    security = ClassSecurityInfo()
    
    def __init__(self, whoami = 'A. Nonyme', subject= '', txt = '', key = '') :
        self.childs = []
        self.subject = subject
        self.content = txt
        self.key = key
        self.author = whoami

    security.declareProtected('Empty comments', 'emptyMe')
    def emptyMe(self) :
        """
        Empty the list of comments
        """
        self.childs = []
        self.content = ''
        self.subject = ''
        self._p_changed = 1

    security.declarePublic('addComment')
    def addComment(self, whoami,
                         subject = 'Pas de sujet', 
                         comment = 'Que du blabla', 
                         father = None) :
        """
        Add a comment to the list.
        If father is omitted (or None), the comment is added to the root of the
        tree.
        If father is present, the comment is added to the comment with this Id.
        """
        if not father or father == self.key :
            key = genKey(self.key, len(self.childs))
            self.childs.append(Comment(whoami, subject, comment, key))
            self._p_changed = 1
            return key
        else :
            for i in self.childs :
                if isChild(i.key, father) : 
                    self._p_changed = 1
                    return i.addComment(whoami, subject, comment, father)
            raise 'FatherKeyError', father

    security.declarePublic('getComment')
    def getComment(self, commentId) :
        """
        Return the comment identified by commentId or an empty comment
        """
        if commentId is None : return Comment()
        if commentId == self.key :
            return self
        elif self.childs :
            for i in self.childs :
                if isChild(i.key, commentId) : return i.getComment(commentId)
        else : return Comment()

    security.declarePublic('flatten')
    def flatten(self, level=0) :
        """
        Just a little function to flatten this comment (for use in zpt).
        Return a liste of tuples. 
         * First element is the nesting level
         * Second element is the comment.
        """
        val = []
        if self.childs :
            for i in self.childs :
                val += i.flatten(level + 1)
        return [(level, self)] + val
    
    security.declarePublic('getChilds')
    def getChilds(self) :
        """Get the childs of the comment"""
        return self.childs

    security.declarePublic('getSubject')
    def getSubject(self) :
        """Get the subject of the comment"""
        return self.subject
        
    security.declarePublic('getText')
    def getText(self) :
        """Get the text of the comment"""
        return self.content

    security.declarePublic('getAuthor')
    def getAuthor(self) :
        """Get the author of the comment"""
        return self.author

    security.declarePublic('getKey')
    def getKey(self) :
        """Get the key of the comment"""
        return self.key

    def __str__(self) :
        return_val = ""
        flat = self.flatten()
        for i in flat :
            return_val += '%s-> CommentId : %s\n' % (i[0] * '    ', i[1].key)
            return_val += '%s   Comment   : %s\n' % (i[0] * '    ', i[1].content)
        return return_val
    
    def __nonzero__(self) :
        return self.content != ''

InitializeClass(Comment)
